using System;
using System.Collections;

public class MyEnumerableType : IEnumerable
{
  private MyEnumeratorType m_myEnumerator;
  private object[]         m_internalArray;
  private int              m_arraySize = 16;
  private int              m_count;

  public MyEnumerableType()
  {
    m_internalArray = new object[m_arraySize];
    m_myEnumerator  = new MyEnumeratorType(this);
  }

  public void Add(object obj)
  {
    m_internalArray[m_count++] = obj;
  }

  public IEnumerator GetEnumerator()
  {
    return m_myEnumerator;
  }

  public int Count
  {
    get
    {
      return m_count;
    }
    set
    {
      m_count = value;
    }
  }

  private class MyEnumeratorType : IEnumerator
  {
    private int m_current = -1;

    private object[]         m_internalArray;
    private MyEnumerableType m_outer;

    public MyEnumeratorType(MyEnumerableType outer)
    {
      m_outer         = outer;
      m_internalArray = outer.m_internalArray;
    }

    public object Current
    {
      get
      {
        Console.WriteLine("Current.");

        if (m_current == -1)
          throw new InvalidOperationException("You tried to access MyEnumerableType when it is empty.");

        return m_internalArray[m_current];
      }
    }

    public bool MoveNext()
    {
      Console.WriteLine("MoveNext.");

      m_current++;

      int m_arraySize = m_internalArray.Length;

      // has array reached max size?
      if (m_arraySize == m_current)
      {
        m_arraySize *= 2;
        object[] temp = new object[m_arraySize];

        m_internalArray.CopyTo(temp, 0);

        m_internalArray = temp;
      }

      bool shouldContinue = m_current == m_outer.Count ? false : true;

      if (!shouldContinue)
        Reset();

      return shouldContinue;
    }

    public void Reset()
    {
      Console.WriteLine("Reset.");
      m_current = -1;
    }
  }
}

class Enumeration
{
  static void Main()
  {
    MyEnumerableType enType = new MyEnumerableType();

    enType.Add(1);
    enType.Add(2);
    enType.Add(3);

    foreach(int obj in enType)
    {
      Console.WriteLine(obj);
    }

    foreach(int obj in enType)
    {
      Console.WriteLine(obj);
    }
    
    Console.ReadLine();
  }
}
